import ctypes
from ctypes import *
from pydbus import SystemBus
from gi.repository import GLib, Gtk, Gio
import gi
from enum import Enum
import syslog

class SoftwareInstallControlMode(Enum):
    CLOSE_MODE = 0
    WHITELIST_MODE = 1
    BLACKLIST_MODE = 2

class SoftwareInstallControlType(Enum):
    WHITELIST = 1
    BLACKLIST = 2

class SoftwareUninstallControlMode(Enum):
    CLOSE_MODE = 0
    OPEN_MODE = 1

class SoftwareInstallControl:
    """
    <node>
        <interface name='com.kylin.softwarectrl.install'>
            <method name='set_install_mode' >
                <arg type='i' direction='in' name='mode'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='get_install_mode' >
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_install_black_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_install_white_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='del_install_black_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
             <method name='del_install_white_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_install_black_package_list' >
                <arg type='as' direction='in' name='package_list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_install_white_package_list' >
                <arg type='as' direction='in' name='package_list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='clear_install_package_list' >
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        try:
            self.bwlist_object = CDLL("libkylin_bwlist.so")
            self._black = create_string_buffer(b"black")
            self._white = create_string_buffer(b"white")
        except Exception:
            syslog.syslog(syslog.LOG_DEBUG, "SoftwareInstallControl init error")

    def set_install_mode(self, mode):
        ret = self.bwlist_object.kylin_bwlist_create()
        syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_create ret = {}".format(ret))
        if ret != -1:
            if mode == SoftwareInstallControlMode.CLOSE_MODE.value:
                ret = self.bwlist_object.kylin_bwlist_set_root_distinguish(0)
                syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_set_root_distinguish(0) ret = {}".format(ret)) 
            elif mode == SoftwareInstallControlMode.WHITELIST_MODE.value:
                ret = self.bwlist_object.kylin_bwlist_set_root_distinguish(1)
                syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_set_root_distinguish(1) ret = {}".format(ret)) 
            elif mode == SoftwareInstallControlMode.BLACKLIST_MODE.value:
                ret = self.bwlist_object.kylin_bwlist_set_root_distinguish(2)
                syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_set_root_distinguish(2) ret = {}".format(ret)) 
            else:
                ret = -1
                syslog.syslog(syslog.LOG_DEBUG, "mode is invalid mode = {}".format(mode))
        return ret

    def get_install_mode(self):
        ret = self.bwlist_object.kylin_bwlist_get_root_distinguish()
        syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_get_root_distinguish ret = {}".format(ret)) 
        return ret

    def add_install_black_package(self, package):
        ret = self.bwlist_object.kylin_bwlist_add(self._black, create_string_buffer(package.encode()))
        syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_add black package = {} ret = {}".format(package, ret)) 
        return ret
    
    def add_install_white_package(self, package):
        ret = self.bwlist_object.kylin_bwlist_add(self._white, create_string_buffer(package.encode()))
        syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_add white package = {} ret = {}".format(package, ret)) 
        return ret

    def del_install_black_package(self, package):
        ret = self.bwlist_object.kylin_bwlist_del(self._black, create_string_buffer(package.encode()))
        syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_del black package = {} ret = {}".format(package, ret)) 
        return ret

    def del_install_white_package(self, package):
        ret = self.bwlist_object.kylin_bwlist_del(self._white, create_string_buffer(package.encode()))
        syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_del white package = {} ret = {}".format(package, ret)) 
        return 

    def add_install_black_package_list(self, package_list):
        for package in package_list:
            ret = self.bwlist_object.kylin_bwlist_add(self._black, create_string_buffer(package.encode()))
            syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_add black package = {} ret = {}".format(package, ret)) 
            if ret == -1:
                return -1;
        return 0;
    
    def add_install_white_package_list(self, package_list):
        for package in package_list:
            ret = self.bwlist_object.kylin_bwlist_add(self._white, create_string_buffer(package.encode()))
            syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_add white package = {} ret = {}".format(package, ret)) 
            if ret == -1:
                return -1;
        return 0;
    
    def clear_install_package_list(self):
        ret = self.bwlist_object.kylin_bwlist_clear()
        syslog.syslog(syslog.LOG_DEBUG, "kylin_bwlist_clear ret = {}".format(ret))
        return ret



class SoftwareUninstallControl:
    """
    <node>
        <interface name='com.kylin.softwarectrl.uninstall'>
            <method name='set_uninstall_mode' >
                <arg type='i' direction='in' name='mode'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='get_uninstall_mode' >
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_uninstall_black_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='del_uninstall_black_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_uninstall_black_package_list' >
                <arg type='as' direction='in' name='package_list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='clear_uninstall_package_list' >
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        try:
            self.udeblist_object = CDLL("libkylin_bwlist.so")
            self._black = create_string_buffer(b"black")
            self._white = create_string_buffer(b"white")
        except Exception:
            syslog.syslog(syslog.LOG_DEBUG, "SoftwareUninstallControl init error")

    def set_uninstall_mode(self, mode):
        ret = self.udeblist_object.kylin_udeblist_create()
        syslog.syslog(syslog.LOG_DEBUG, "kylin_udeblist_create ret = {}".format(ret))
        if ret != -1:
            if mode == SoftwareUninstallControlMode.CLOSE_MODE.value:
                ret = self.udeblist_object.kylin_udeblist_set_root_distinguish(0)   #不管控
                syslog.syslog(syslog.LOG_DEBUG, "kylin_udeblist_set_root_distinguish(0) ret = {}".format(ret))
            elif mode == SoftwareUninstallControlMode.OPEN_MODE.value:
                ret = self.udeblist_object.kylin_udeblist_set_root_distinguish(2)   #黑名单
                syslog.syslog(syslog.LOG_DEBUG, "kylin_udeblist_set_root_distinguish(1) ret = {}".format(ret))
            else:
                ret = -1
                syslog.syslog(syslog.LOG_DEBUG, "mode is invalid mode = {}".format(mode))
        return ret

    def get_uninstall_mode(self):
        ret = self.udeblist_object.kylin_udeblist_get_root_distinguish()
        syslog.syslog(syslog.LOG_DEBUG, "kylin_udeblist_get_root_distinguish ret = {}".format(ret))
        if ret == 2: # 2为黑名单
            return 1
        return ret

    def add_uninstall_black_package(self, package):
        ret = self.udeblist_object.kylin_udeblist_add(self._black, create_string_buffer(package.encode()))
        syslog.syslog(syslog.LOG_DEBUG, "kylin_udeblist_add black package = {} ret = {}".format(package, ret))
        return ret

    def del_uninstall_black_package(self, package):
        ret = self.udeblist_object.kylin_udeblist_del(self._black, create_string_buffer(package.encode()))
        syslog.syslog(syslog.LOG_DEBUG, "kylin_udeblist_del black package = {} ret = {}".format(package, ret))
        return ret

    def add_uninstall_black_package_list(self, package_list):
        for package in package_list:
            ret = self.udeblist_object.kylin_udeblist_add(self._black, create_string_buffer(package.encode()))
            syslog.syslog(syslog.LOG_DEBUG, "kylin_udeblist_add black package = {} ret = {}".format(package, ret))
            if ret == -1:
                return -1;
        return 0;

    def clear_uninstall_package_list(self):
        ret = self.udeblist_object.kylin_udeblist_clear()
        syslog.syslog(syslog.LOG_DEBUG, "kylin_udeblist_clear ret = {}".format(ret))
        return ret


if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.softwarectrl", SoftwareInstallControl(), SoftwareUninstallControl())
    loop.run()